<?php

namespace Drupal\url_access_control\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Url;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Event subscriber to restrict access to specific URLs.
 */
class UrlAccessControlSubscriber implements EventSubscriberInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new UrlAccessControlSubscriber.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Subscribe to the KernelEvents::REQUEST event with priority 30.
    $events[KernelEvents::REQUEST][] = ['onKernelRequest', 30];
    return $events;
  }

  /**
   * Responds to kernel request events.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The event to process.
   */
  public function onKernelRequest(RequestEvent $event) {
    $request = $event->getRequest();
    $path = $request->getPathInfo();
    $current_route = $request->attributes->get('_route');

    // Load configuration.
    $config = $this->configFactory->get('url_access_control.settings');
    $urls = $config->get('urls') ?? [];

    $site_config = $this->configFactory->get('system.site');
    $default_403_path = $site_config->get('page.403');
    // Define paths and routes to exclude.
    $excluded_paths = [
      $default_403_path ?? '/403',
      '/url-access-denied',
    ];
    $excluded_routes = [
      'url_access_control.access_denied',
      'system.403',
    ];

    // Skip access check for excluded paths or routes.
    if (in_array($path, $excluded_paths, TRUE) || in_array($current_route, $excluded_routes, TRUE)) {
      return;
    }
 
    // Check if the current path starts with any restricted URL.
    foreach ($urls as $restricted_url) {
      $restricted_url = '/' . ltrim($restricted_url, '/');
      if ($restricted_url && strpos($path, $restricted_url) === 0) {
        $referer = $request->headers->get('referer');
        if ($referer) {
          return; // Allow if Referer is present.
        }
        // Build redirect URL to 403 or custom denied page.
        if ($default_403_path) {
          $page_403_url = Url::fromUri('internal:' . $default_403_path)
          ->setAbsolute()
          ->toString();
        }
        else {
          $page_403_url = Url::fromRoute('url_access_control.access_denied')
            ->setAbsolute()
            ->toString();
        }
        
        // Set redirect response.
        $redirect_response = new RedirectResponse($page_403_url);
        $event->setResponse($redirect_response);
        return;
      }
    }
  }

}
